######################################################################
######## Bootstrap of the counterfactuals  ###########################
######################################################################
setwd("G:/QXG/R/CMX2018QE/Bootstrap")

# load the functions needed  
source("CMX_Proc_Choice.R")
source("CMX_Proc_FUNcountsys.R")
source("CMX_Proc_Resim.R")
source("CMX_Proc_Sucprob.R")
source("CMX_Proc_ARCboot.R")
source("CMX_Proc_TCboot.R")
source("CMX_Proc_TMboot.R")
###################################################################################################
#####   Parameters and vectors used for counterfactual simulations ################################
# PARgrade : all the parameter from the grade equation estimation 
#     PARgrade[[1]]: the parameters of the first grade equation (vector of size 12)
#     PARgrade[[2]]: the parameters of the second grade equation (vector of size 9)
#     PARgrade[[3]]: Control function parameter in the ranking combination (a number)
#
# ubootsamp: the empirical residuals of grade equations, choices and Enem scores
#     ubootsamp[[1]]:  u1 hat, the residuals of the first grade equation (vector of size nobs)
#     ubootsamp[[2]]:  u2 hat, the residuals of the second grade equation (vector of size nobsm2)
#     ubootsamp[[3]]:  the observed choice, 1 means choosing Sobral (vector of size nobs)
#     ubootsamp[[4]]:  the observed Enem scores (vector of size nobs)
#
# datagrade1: all the covariates of the first grade equation  (matrix of size nobs*11)
# datagrade2: all the covariates of the second grade equation (matrix of size nobs*7)
#
# PARpref: the parameters of the preference estimation and the boundaries for bootstraps
#     PARpref[[1]]: the paramters of the preference estimation (vector of size 11)
#     PARpref[[2]]: the upper bound of parameters (vector of size 11)
#     PARpref[[3]]: the lower bound of parameters (vector of size 11)
#
# vecpref: some results from the preference estimation 
#     vecpref[[1]]: log utility difference between Sobral and Fortaleza (vector of size nobs)
#     vecpref[[2]]: the proabailities of being one particular type of students: Deltas (matrix nobs*3)
#     vecpref[[3]]: the expected success probabilities in the original system at both schools (matrix nobs*2)
#
# datapref: all the covariates of the preference estimation (matrix of size nobs*6)
################################################################################################
#setwd("E:/R works/CMX code")
load("CMX_Prep_boot.rdata")
load("CMX_Bootstrap_Matrix.rdata")
PARgrade   <- prep_boot[[1]]
ubootsamp  <- prep_boot[[2]]
datagrade1 <- prep_boot[[3]]
datagrade2 <- prep_boot[[4]]
PARpref    <- prep_boot[[5]]
vecprefboot    <- prep_boot[[6]]
datapref   <- prep_boot[[7]]
mft        <- prep_boot[[8]]

# number of observations and number of final seats
nobs<- 2867
nos <- 40
nof <- 150 

#      Number of Variables 
ndt <- 2       # Number of variables in delta and delta1 
na <- 7       # Number of variables in alpha   
lam <- 5.e-4   # Penalization parameter

nsimT <- 199     #  number of system resimulations in each bootstrap
nsim  <- 499     #  number of simulations in the success probability computation
nboot <- 499      #  number of bootstraps 

# boostrap of the admission rate change
library(doParallel)
cl <- makeCluster(24)   # set the number of cpu 
registerDoParallel(cl)

Bootmatrix <- m



BootTC <- foreach(b = (1:nboot), .combine="cbind", .packages="doParallel" ) %dopar%
{
  TCboot(b, Bootmatrix, prep_boot, 199)  
}

save(BootTC, file="BootTC.rdata")



BootARC <- foreach(b = (1:nboot), .combine="cbind") %dopar%
{
  ARCboot(b, Bootmatrix, prep_boot, 199)  
}

save(BootARC, file="BootARC.rdata")


BootTM <- foreach(b = (1:nboot), .combine="cbind") %dopar%
{
  TMboot(b, Bootmatrix, prep_boot, 199)  
}

save(BootTM, file="BootTM.rdata")


stopCluster(cl)

#### ============================= Table 3 ===========================================================
#
Table_3 <- rbind(apply(REsim[[1]], 1, mean),
                 apply(REsim[[1]], 1, sd),
                 apply(BootARC, 1, mean)[1:4],
                 apply(BootARC, 1, mean)[5:8],
                 apply(BootTC, 1, mean)[1:4],
                 apply(BootTC, 1, mean)[5:8],
                 apply(BootTM, 1, mean)[1:4],
                 apply(BootTM, 1, mean)[5:8])
colnames(Table_3) <- c("Sobral 1st stage" ,"Fortaleza 1st stage","Sobral 2nd stage" ,"Fortaleza 2nd stage")
rownames(Table_3) <- c("OG mean", "OG sd","ARC mean", "ARC sd","TC mean", "TC sd","TM mean", "TM sd")

print(Table_3, digits=3)




