####
BOOTALL<-function(PARgrade,PARpref,vecpref,uboot,e1,e2,u1,u2, data1,data2,datapref)
{
  # creating the bootstrap samples
  # load the parameters 
  beta1 <- PARgrade[[1]]
  beta2 <- PARgrade[[2]]
  cfu1 <- PARgrade[[3]]
  nobs <- length(u1)
  # grades
  m1new <- as.vector(as.matrix(cbind(rep(1,nobs), data1)) %*% beta1) + u1
  m2new <- as.vector(as.matrix(cbind(rep(1,nobs),data2)) %*% beta2[1:8]) + u1*beta2[9]+u2
  
  # choices
  # using the mean of the success probability in the original system
  # but the random draws of epsilons do not need to depend on the observed choices.
  choixboot <- FUNchoiceboot(vecpref, e1, e2)
  
  # generate the thresholds
  # the combination for ranking at the first stage
  m1sk <- m1new + 120*uboot[[4]]/63
  # the combination for ranking
  m2sk <- 0.4*m1sk + 0.6*m2new
  # first stage thresholds
  t1s <- sort(m1sk[choixboot==1], decreasing=TRUE)[nos*4]
  t1f <- sort(m1sk[choixboot==0], decreasing=TRUE)[nof*4]
  # second stage thresholds
  t2s <- sort(m2sk[choixboot==1 & m1sk>=t1s], decreasing=TRUE)[nos] 
  t2f <- sort(m2sk[choixboot==0 & m1sk>=t1f], decreasing=TRUE)[nof] 
  # thresholds of all
  tsimboot <- c(t1s, t1f, t2s, t2f)
  
  # censored the students who fail the first exam
  select <- rep(0,nobs)
  select[choixboot==1 & m1sk >= t1s] <- 1
  select[choixboot==0 & m1sk >= t1f] <- 1
  # the subsample for second stage grade estimation
  dataselect <- cbind(m2new[select==1], data2[select==1,], u1[select==1])
  colnames(dataselect)[c(1,9)] <- c("m2new","u1")
  # re-estimate the grade equations
  lm1new <- lm(m1new ~ female+age+em_supl+em_priv+prep+repet+CFF+bseps1+bseps2+CFFeps+educm, data=data1)
  lm2new <- lm(m2new ~ female+age+em_supl+em_priv+prep+repet+epsdat+u1, data=dataselect)
  # new betas
  beta1new <- coefficients(lm1new)
  beta2new <- coefficients(lm2new)
  # new fitte grades
  m1boot <- as.vector(as.matrix(cbind(rep(1,nobs), data1)) %*% beta1new)
  m2boot <- as.vector(as.matrix(cbind(rep(1,nobs),data2)) %*% beta2new[1:8])
  
  # resimulate the system with new betas 
  databoot <- cbind(choixboot, m1boot, m2boot)       # data for resimulation
  simulT <- matrix(runif(nobs*nsimT), nrow = nobs)   # random draw from uniform distribution
  
  # using the same resimulation subroutine to have the distribution of thresholds and success probability
  SYSboot <- FUNresimboot(databoot,nsimT,uboot,simul,PARgrade)
  Tboot <- SYSboot[[1]]                      # mean of thresholds
  P2Sobboot  <- apply(SYSboot[[3]], 1, mean) # mean of success probability at Sobral
  P2Fortboot <- apply(SYSboot[[5]], 1, mean) # mean of success probabiltiy at Fortaleza
  
  # for the threhsolds 
  Tbootmean <- apply(Tboot, 2, mean)
  Tbootvar <- apply(Tboot, 2, var) 
  
  # log probability difference  
  wboot <- log(P2Sobboot/P2Fortboot)

  #  the preemption of the preference estimation
  ndt <- 2       # Number of variables in delta and delta1 
  na <- 7        # Number of variables in alpha   
  lam <- 5.e-4   # Penalization parameter
  
  # data for preference estimation
  dprefboot <- cbind(choixboot,wboot,datapref)
  dtaboot <- as.matrix(dprefboot)[,c(1:2,3,3:8)]
  
  # functions for bootstrap the preferences
  LIKboot <- function(coef) slikTPboot(coef,dtaboot)
  gradLIKboot <- function(coef) gradlikTPboot(coef,dtaboot)
  
  # starting values and their bounds
  coef0boot<-PARpref[[1]]*1.05
  ubnd<-PARpref[[2]]
  lbnd<-PARpref[[3]]
  
  #  Estimation of preference  
  valbl<-nloptr(coef0boot, eval_f = LIKboot, eval_grad_f = gradLIKboot, lb=lbnd, ub=ubnd,
                opts = list("algorithm" = "NLOPT_LD_LBFGS", "print_level" = 2,  "xtol_rel" = 1.0e-6,
                            "maxtime"= 1000, "maxeval" = 200))
  bprefboot<-valbl$solution
  
  #  new deltas  
  delta10 <- 1/(1+exp(-cbind(rep(1,nobs), datapref[,1]) %*% bprefboot[1:ndt]))
  delta11 <- (1-delta10)/(1+exp(-cbind(rep(1,nobs), datapref[,1]) %*% bprefboot[(ndt+1):(2*ndt)]))
  delta01 <- 1-delta10 -delta11
  
  DTboot <- cbind(delta10, delta11, delta01)
  # the re-estimated preference parameters
  return(list(beta1new, beta2new, bprefboot, DTboot, choixboot, Tbootmean, Tbootvar))
}


