############################################################################################
####   likelihood function and gradient functions for the bootstrap ########################
############################################################################################

# Likelihood function 
likTPboot <- function(coefboot, dtaboot) {
  
  #      Number of Variables 
  nobs<-dim(dtaboot)[1]
  ndt <- 2       # Nb of variables in delta and delta1 
  na <- 7       # Nb of variables in alpha   
  lam <- 5.e-4   # Penalization parameter
  
  delta1 <- 1/(1+exp(-cbind(rep(1,nobs), dtaboot[,3:(ndt+1)]) %*% coefboot[1:ndt]))
  delta <- (1-delta1)/(1+exp(-cbind(rep(1,nobs), dtaboot[,3:(ndt+1)]) %*% coefboot[(ndt+1):(2*ndt)]))
  arg <- cbind(rep(1,nobs), dtaboot[,(ndt+2):(ndt+na)]) %*% coefboot[(2*ndt+1):(2*ndt+na)]
  
  p <- delta1 + delta*pnorm(arg + dtaboot[,2]) 
  
  dtaboot[,1]*log(p) + (1-dtaboot[,1])*log(1-p) - lam*sum(coefboot^2)/2
}

slikTPboot <- function(coefboot, dtaboot) sum(-likTPboot(coefboot,dtaboot))

# Likelihood gradient
gradlikTPeboot <- function(coefboot,dtaboot) {
  #      Number of Variables 
  nobs<-dim(dtaboot)[1]
  #      Number of Variables 
  ndt <- 2       # Nb of variables in delta and delta1 
  na <- 7       # Nb of variables in alpha   
  lam <- 5.e-4   # Penalization parameter
  
  lind <- list(1:ndt, ndt+(1:ndt), 2*ndt+(1:na))
  ldtaboot <- list(cbind(rep(1,nobs), dtaboot[,3:(ndt+1)]), cbind(rep(1,nobs), dtaboot[,3:(ndt+1)]), 
                   cbind(rep(1,nobs), dtaboot[,(ndt+2):(ndt+na)]))
  
  # Parameters in each index 
  narg <- paste("likTPboot", "(", "c", "(", sep="")
  pderiv <- NULL
  
  m <- 0
  while (m < 3) {
    m <- m + 1
    
    nom <- paste("c", m, sep="")
    assign(nom, coefboot[lind[[m]]][1])
    narg <- paste(narg, nom, ",", sep="") 
    pderiv <- c(pderiv, nom)
    
    nom <- paste("cr", m, sep="")
    assign(nom, coefboot[lind[[m]]][-1])
    
    if (m<3) narg <- paste(narg, nom, ",", sep="") else narg <- paste(narg, nom, ")", sep="")
    
  }
  narg<-paste(narg,",","dtaboot",")")
  # Principal gradient 
  g <- numericDeriv(quote(eval(parse(text=narg))), pderiv)
  grad <- attr(g,"gradient")
 
  # Gradient recomposition
  m <- 0
  grf <- NULL
  while (m < 3) {
    m <- m + 1
    grf <- cbind(grf, (grad[,m] + lam*coefboot[lind[[m]]][1])*ldtaboot[[m]] 
                 - lam * rep(1,nobs) %o% coefboot[lind[[m]]])
  }
  
  grf
  
}  

gradlikTPboot <- function(coefboot, dtaboot) apply(-gradlikTPeboot(coefboot,dtaboot), 2, sum)
