# Resimulate the system with random draws of grade shocks u1 and u2
FUNresim<-function(x,nsimT,uboot,simul,PARgrade)
{
  # choices are decided by the sample observation we only simulate for the observed sample
  #simulation numbers
  
  nobs <- length(uboot[[1]])
  nos <- 40
  nof <- 150
  choice <- x[,1]
  m1h <- x[,2]
  m2h <- x[,3]
  Enem <- uboot[[4]]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  u1<-uboot[[1]]
  u2<-uboot[[2]]
  # use parallel computing to do the simulations
  # simulate the thresholds first
   tsim<-foreach(i=(1:nsimT), .export=c("Enem"), .combine="cbind") %dopar%
  { 
    u1n<-sample(u1,size=nobs,replace=TRUE)
    #u2n<-sample(u2,size=nobs, replace=TRUE) # the sampling is larger than the sample
    u2n<-sample(u2,size=nobs,replace=TRUE)
    # the simulated grades
    m1s<-m1h+u1n
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1n+u2n
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # divide the population by their choices in the sample
    t1s<-sort(m1sk[choice==1], decreasing=TRUE)[nos*4]
    t1f<-sort(m1sk[choice==0], decreasing=TRUE)[nof*4]
    
    t2s<-sort(m2sk[choice==1 & m1sk>=t1s], decreasing=TRUE)[nos] 
    t2f<-sort(m2sk[choice==0 & m1sk>=t1f], decreasing=TRUE)[nof] 
    
    c(t1s, t1f, t2s, t2f)
  }
  # simulate the success probabilities 
  PS<-foreach(i=(1:nsimT), .export=c("Enem","sucprob"), .combine="cbind") %dopar%
         {
           #  Individual invariants 
           bm10 <- rep(1,nobs) %o% c(tsim[1,i],tsim[2,i])  - 120*Enem/63 - m1h
           bm20 <- rep(1,nobs) %o% c(tsim[3,i],tsim[4,i])/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
           # for sobral
           sucsob<-sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1)
           # for fortaleza
           sucfort<-sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1)
           #get them together
           c(sucsob[,1],sucsob[,2],sucfort[,1],sucfort[,2])
         }
  # the outputs
  return(list(tsim,PS[(1+nobs):(nobs*2),],PS[(1+nobs*3):(nobs*4),]))
}

# resimulation with bootstrap
#for bootstraping the preferences 
FUNresimboot<-function(x,nsimT,uboot,simul,PARgrade)
{
  # choices are decided by the sample observation we only simulate for the observed sample
  #simulation numbers 
  nobs<-length(uboot[[1]])
  nos<-40
  nof<-150
  choice<-x[,1]
  m1h<-x[,2]
  m2h<-x[,3]
  Enem<-uboot[[4]]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  u1<-uboot[[1]]
  u2<-uboot[[2]]
  
  tsim<-matrix(rep(0), nrow=nsimT, ncol=4)
  P1Sob<-matrix(rep(0),nrow=nobs, ncol=nsimT)
  P1Fort<-matrix(rep(0),nrow=nobs, ncol=nsimT)
  P2Sob<-matrix(rep(0),nrow=nobs, ncol=nsimT)
  P2Fort<-matrix(rep(0),nrow=nobs, ncol=nsimT)
  
  for (i in 1:nsimT)
  {
    u1n<-sample(u1,size=nobs,replace=TRUE)
    #u2n<-sample(u2,size=nobs, replace=TRUE) # the sampling is larger than the sample
    u2n<-sample(u2,size=nobs,replace=TRUE)
    # the simulated grades
    m1s<-m1h+u1n
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1n+u2n
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # divide the population by their choices in the sample
    t1s<-sort(m1sk[choice==1], decreasing=TRUE)[nos*4]
    t1f<-sort(m1sk[choice==0], decreasing=TRUE)[nof*4]
    
    t2s<-sort(m2sk[choice==1 & m1sk>=t1s], decreasing=TRUE)[nos] 
    t2f<-sort(m2sk[choice==0 & m1sk>=t1f], decreasing=TRUE)[nof] 
    tsim[i,]<-c(t1s, t1f, t2s, t2f)
    #  Individual invariants 
    bm10 <- rep(1,nobs) %o% c(t1s,t1f)  - 120*Enem/63 - m1h
    bm20 <- rep(1,nobs) %o% c(t2s,t2f)/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
    #    Success probabilities 
    psboot <- cbind( sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1), sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1))
    P1Sob[,i]<-psboot[,1]
    P2Sob[,i]<-psboot[,2]
    P1Fort[,i]<-psboot[,3]
    P2Fort[,i]<-psboot[,4]
  }
  return(list(tsim,P1Sob,P2Sob,P1Fort,P2Fort))
}


# resimualtion function of admission rate change counterfactual
#for bootstraping the preferences 
FUNresimARC<-function(x,nsimT,uboot,simul,PARgrade)
{
  # choices are decided by the sample observation we only simulate for the observed sample
  #simulation numbers 
  nobs <- length(uboot[[1]])
  nos <- 40
  nof <- 150
  choix <- x[[1]]
  m1h <- x[[2]]
  m2h <- x[[3]]
  Enem <- uboot[[4]]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  u1 <- uboot[[1]]
  u2 <- uboot[[2]]

  tsim<-matrix(rep(0), nrow=nsimT, ncol=4)
  for (i in 1:nsimT)
  {
  u1n<-sample(u1,size=nobs,replace=TRUE)
  #u2n<-sample(u2,size=nobs, replace=TRUE) # the sampling is larger than the sample
  u2n<-sample(u2,size=nobs,replace=TRUE)
  # the simulated grades
  m1s<-m1h+u1n
  # the combination for ranking
  m1sk<-m1s+120*Enem/63
  # the second stage grades 
  m2s<-m2h+cfu1m2*u1n+u2n
  # the combination for ranking
  m2sk<-0.4*m1sk +0.6*m2s
  # divide the population by their choices in the sample
  t1s<-sort(m1sk[choix==1], decreasing=TRUE)[nos*2]
  t1f<-sort(m1sk[choix==0], decreasing=TRUE)[nof*2]
  
  t2s<-sort(m2sk[choix==1 & m1sk>=t1s], decreasing=TRUE)[nos] 
  t2f<-sort(m2sk[choix==0 & m1sk>=t1f], decreasing=TRUE)[nof] 
  
  tsim[i,] <- c(t1s, t1f, t2s, t2f)
}
  return(tsim)
}


# resimulation function of two choice counterfactual 
FUNresimTC <- function(x,nsimT,uboot,simul,PARgrade)
{
  #simulation numbers 
  nobs <- length(uboot[[1]])
  nos <- 40
  nof <- 150
  choix <- x[[1]]
  m1h <- x[[2]]
  m2h <- x[[3]]
  Enem <- uboot[[4]]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  u1 <- uboot[[1]]
  u2 <- uboot[[2]]
  
  tsim<-matrix(rep(0), nrow=nsimT, ncol=4)
  for (i in 1:nsimT)
  {
    u1n<-sample(u1,size=nobs,replace=TRUE)
    #u2n<-sample(u2,size=nobs, replace=TRUE) # the sampling is larger than the sample
    u2n<-sample(u2,size=nobs,replace=TRUE)
  
    sobonly<-choix[,1]
    fortonly<-choix[,2]
    dsf<-choix[,3]
    dfs<-choix[,4]
    # the simulated grades
    m1s<-m1h+u1n 
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1n +u2n 
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # divide the population by their choices in the sample
    # chances are that the people chose Sobral is small than the admission number
    t1sk<-sort(m1sk[sobonly==1 | dsf==1], decreasing=TRUE)
    t1s<- max(t1sk[nos*4], min(t1sk),na.rm=TRUE)
    t1fk<-sort(m1sk[fortonly==1 | dfs==1], decreasing=TRUE) 
    t1f<- max(t1fk[nof*4], min(t1sk),na.rm=TRUE)
    # refinement of the thresholds. 
    if (t1s > t1f){
      #take those who fail sobral when choosing (s,f)
      nrankfort<-c(m1sk[fortonly==1 | dfs==1], m1sk[dsf==1 & m1sk<t1s])
      nrankfort<-sort(nrankfort, decreasing=TRUE)
      t1f<-max(nrankfort[nof*4], min(nrankfort),na.rm=TRUE)
    }else{
      #take those who fail sobral when choosing (f,s)
      nranksob<-c(m1sk[sobonly==1 | dsf==1 ], m1sk[dfs==1 & m1sk<t1f])
      nranksob<-sort(nranksob, decreasing=TRUE)
      t1s<-max(nranksob[nos*4], min(nranksob),na.rm=TRUE)
    }
    #sob
    sob2t<-ifelse(sobonly==1& m1sk>= t1s, 1, 0)
    sob22t<-ifelse(dsf==1 & m1sk>=t1s,1,0)
    sob222t<- ifelse(dfs==1 & m1sk>=t1s & m1sk<t1f,1,0)
    sum(sob22t+sob2t+sob222t)
    sobtwo<-sob22t+sob2t+sob222t
    #fort
    fort2t<-ifelse(fortonly==1 & m1sk >= t1f, 1, 0)
    fort22t<-ifelse(dfs==1 & m1sk>=t1f,1,0)
    fort222t<-ifelse(dsf ==1 & m1sk>=t1f & m1sk<t1s, 1, 0)
    sum(fort222t+fort22t+fort2t)
    forttwo<- fort222t+fort22t+fort2t
    
    # second stage
    t2sk<-sort(m2sk[sobtwo==1], decreasing=TRUE)
    t2s<- max(t2sk[nos], min(t2sk), na.rm=TRUE) 
    t2fk<-sort(m2sk[forttwo==1], decreasing=TRUE)
    t2f<- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    
    tsim[i,] <- c(t1s, t1f, t2s, t2f)
}
  # simulate the success probabilities 
#  PS<-foreach(i=(1:nsimT), .export=c("Enem","sucprob"), .combine="cbind") %dopar%
#{
  #  Individual invariants 
#  bm10 <- rep(1,nobs) %o% c(tsim[1,i],tsim[2,i])  - 120*Enem/63 - m1h
#  bm20 <- rep(1,nobs) %o% c(tsim[3,i],tsim[4,i])/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
  # for sobral
#  sucsob<-sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1)
  # for fortaleza
#  sucfort<-sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1)
  #get them together
#  c(sucsob[,1],sucsob[,2],sucfort[,1],sucfort[,2])
#}
  # the outputs
  #return(list(tsim,PS[(1+nobs):(nobs*2),],PS[(1+nobs*3):(nobs*4),]))
  return(tsim)
}


# resimulate the timing change counterfactual system.
FUNresimTM <- function(x,nsimT,uboot,simul,PARgrade)
{
  # simulation numbers, parameters and vectors 
  nobs <- length(uboot[[1]])
  nos <- 40
  nof <- 150
  Choix <- x[[1]]
  m1h <- x[[2]]
  m2h <- x[[3]]
  Enem <- uboot[[4]]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  u1 <- uboot[[1]]
  u2 <- uboot[[2]]
  
  tsim<-matrix(rep(0), nrow=nsimT, ncol=4)
  for (i in 1:nsimT)
  { 
    u1n <- sample(u1, size=nobs, replace=TRUE)
    u2n <- sample(u2, size=nobs, replace=TRUE)  
    # the simulated grades
    m1s <- m1h + u1n
    # the combination for ranking
    m1sk <- m1s + 120*Enem/63
    # the second stage grades 
    m2s <- m2h + cfu1*u1n + u2n 
    # the combination for ranking
    m2sk <- 0.4*m1sk + 0.6*m2s
    
    # one by one decision 
    Nsob  <- 0
    Nfort <- 0
    ChoixTM <- rep(0,nobs)
    # all the variables 
    mk <- cbind(m1sk, m2sk, Choix)
    mk <- mk[order(mk[,1], decreasing=TRUE), ]  # sort the grade wrt m1
    
    for (j in 1:nobs){
      # one by one decision making
      if (Nsob < nos*4 & Nfort < nof*4){
        # proc 1
        ChoixTM[j] <- Choix[j,1]
      }
      if (Nsob < nos*4 & Nfort == nof*4){
        #proc 2
        if (Choix[j,1]==1){
          ChoixTM[j] <- Choix[j,1]
        }
        else{
          ChoixTM[j] <- Choix[j,2]
        } 
      }
      if (Nsob == nos*4 & Nfort < nof*4){
        # proc 3
        if (Choix[j,1] == 1){
          ChoixTM[j] <- Choix[j,2]
        }
        else{
          ChoixTM[j] <- Choix[j,1]
        } 
      }
      if (Nsob == nos*4 & Nfort == nof*4){
        # proc 4
        ChoixTM[j] <- NA
      }  
      
      # addition of Nsob
      if (is.na(ChoixTM[j]) == FALSE){
        Nsob <- Nsob + ChoixTM[j]
      }else{
        Nsob <- Nsob
      }
      # addition of Nfort
      if (is.na(ChoixTM[j]) == FALSE){
        Nfort <- Nfort + (1 - ChoixTM[j])
      }else{
        Nfort <- Nfort
      }
      
    }
    
    # get the thresholds 
    # for sobral 
    t1sk <- sort(mk[ChoixTM==1,1], decreasing=TRUE)
    t1s  <- max(t1sk[nos*4], min(t1sk), na.rm=TRUE)
    t2sk <- sort(mk[ChoixTM==1,2], decreasing=TRUE)
    t2s  <- max(t2sk[nos], min(t2sk), na.rm=TRUE)
    # for fortaleza
    t1fk <- sort(mk[ChoixTM==0,1], decreasing=TRUE)
    t1f  <- max(t1fk[nof*4], min(t1fk), na.rm=TRUE)
    t2fk <- sort(mk[ChoixTM==0,2], decreasing=TRUE)
    t2f  <- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    
    tsim[i,] <- c(t1s, t1f, t2s, t2f)
  }
  return(tsim)
}