##############################################################################
#################   Two choice counterfactuals ###############################
##############################################################################
# load the functions necessary 
source("CMX_Proc_Choice.R")
source("CMX_Proc_FUNcountsys.R")
source("CMX_Proc_Resim.R")
source("CMX_Proc_Count_TM.r")
library(doParallel)
cl <- makeCluster(28)  # set the number of cpu you want to use
registerDoParallel(cl)

###################################################################################################
#####   Parameters and vectors used for counterfactual simulations ################################
# PARgrade : all the parameter from the grade equation estimation 
#     PARgrade[[1]]: the parameters of the first grade equation (vector of size 12)
#     PARgrade[[2]]: the parameters of the second grade equation (vector of size 9)
#     PARgrade[[3]]: Control function parameter in the ranking combination (a number)
#
# ubootsamp: the empirical residuals of grade equations, choices and Enem scores
#     ubootsamp[[1]]:  u1 hat, the residuals of the first grade equation (vector of size nobs)
#     ubootsamp[[2]]:  u2 hat, the residuals of the second grade equation (vector of size nobsm2)
#     ubootsamp[[3]]:  the observed choice, 1 means choosing Sobral (vector of size nobs)
#     ubootsamp[[4]]:  the observed Enem scores (vector of size nobs)
#
# datagrade1: all the covariates of the first grade equation  (matrix of size nobs*11)
# datagrade2: all the covariates of the second grade equation (matrix of size nobs*7)
#
# PARpref: the parameters of the preference estimation and the boundaries for bootstraps
#     PARpref[[1]]: the paramters of the preference estimation (vector of size 11)
#     PARpref[[2]]: the upper bound of parameters (vector of size 11)
#     PARpref[[3]]: the lower bound of parameters (vector of size 11)
#
# vecpref: some results from the preference estimation 
#     vecpref[[1]]: log utility difference between Sobral and Fortaleza (vector of size nobs)
#     vecpref[[2]]: the proabailities of being one particular type of students: Deltas (matrix nobs*3)
#     vecpref[[3]]: the expected success probabilities in the original system at both schools (matrix nobs*2)
#
# datapref: all the covariates of the preference estimation (matrix of size nobs*6)
################################################################################################

#   set the simulation parameters
set.seed(1894285437, kind = NULL)

load("CMX_Prep_boot.rdata")
PARgrade   <- prep_boot[[1]]
ubootsamp  <- prep_boot[[2]]
datagrade1 <- prep_boot[[3]]
datagrade2 <- prep_boot[[4]]
PARpref    <- prep_boot[[5]]
vecpref    <- prep_boot[[6]]
datapref   <- prep_boot[[7]]
mft        <- prep_boot[[8]]

# number of observations and number of final seats
nobs<- length(vecpref[,1])
nos <- 40
nof <- 150 

nsimT <- 499     #  number of system resimulations in each bootstrap
nsim  <- 999     #  number of simulations in the success probability computation

#  random draw from uniform distribution for the success probability computation
simul <- matrix(runif(nobs*nsim), nrow = nobs)     


nsimul <- 10    # number random draws for the converging algorithm
#  preference shock, once drawn, fix across all the iterations 
eps1ct <-  runif(nobs)     #----- random draws for choice proportions
eps2ct <-  runif(nobs)     #----- random draws for preference difference



# simulate the timing change counterfactual
e1<-eps1ct
e2<-eps2ct


NobTM <- 499
uboot <- ubootsamp

nresim <- 4999
smallnumber <- 0.001
R <- 4
probSIM <- cbind(vecpref[,5], vecpref[,6])
cfu1 <- PARgrade[[3]]
cfu1m2 <- PARgrade[[2]][9]
m1h <- mft[,1]
m2h <- mft[,2]
Enem <- uboot[[4]]

PSob <- matrix(rep(0),nrow=nobs, ncol=nsimul)
PFort <- matrix(rep(0),nrow=nobs, ncol=nsimul)
tsim <- matrix(rep(0), nrow=nsimul, ncol=4)
SobSIM <- matrix(rep(0),nrow=nobs, ncol=nsimul)
convg <- rep(0, nsimul)

# start the iteration 
for (i in 1:nsimul)
{ 
  Choix<-FUNchoiceTM(e1,e2,vecpref, probSIM, uboot)
  
  u1 <- foreach(j=(1:nresim), .combine="cbind") %dopar%
  {
    u1x <- sample(uboot[[1]], size = nobs, replace=FALSE)
  }
  
  u2 <- foreach(j=(1:nresim), .combine="cbind") %dopar%
  {
    u2x <- sample(uboot[[2]], size = nobs, replace=TRUE)
  }
  
  # simulate the success probabilities 
  ps<-foreach(k=(1:nresim), .combine="cbind") %dopar%
  {
    
    # the simulated grades
    m1s<-m1h+u1[,k]  
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1[,k] + u2[,k]  
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # one by one decision 
    Nsob<-0
    Nfort<-0
    ChoixTM<-rep(0,nobs)
    # all the variables 
    mk<-cbind(m1sk, m2sk,Choix)
    mk<-mk[order(mk[,1], decreasing=TRUE), ]  # sort the grade wrt m1
    
    for (j in 1:nobs){
      #one by one decision making
      if (Nsob<nos*4 & Nfort<nof*4){
        # proc 1
        ChoixTM[j]<-mk[j,3]
      }
      
      if (Nsob<nos*4 & Nfort == nof*4){
        #proc 2
        if (mk[j,3]==1){
          ChoixTM[j]<-mk[j,3]
        }
        if (mk[j,3]==0){
          ChoixTM[j]<-mk[j,4]
        }
      }
      
      if (Nsob==nos*4 & Nfort < nof*4){
        # proc 3
        if (mk[j,3]==0){
          ChoixTM[j]<-mk[j,3]
        }
        if (mk[j,3]==1){
          ChoixTM[j]<-mk[j,4]
        }
      }
      
      if (Nsob==nos*4 & Nfort==nof*4){
        # proc 4
        ChoixTM[j]<-NA
      } 
      
      ChoixTM[j]
      # addition of Nsob
      if (is.na(ChoixTM[j])==FALSE){
        Nsob<-Nsob+ChoixTM[j]
      }else{
        Nsob<-Nsob
      }
      # addition of Nfort
      if (is.na(ChoixTM[j])==FALSE){
        Nfort<-Nfort+(1-ChoixTM[j])
      }else{
        Nfort<-Nfort
      }
      
    }
    # the final program choice
    SobSIM[,i]<-ChoixTM
    
    # second stage
    t1sk<-sort(mk[ChoixTM==1,1], decreasing=TRUE)
    t1s<-max(t1sk[nos*4], min(t1sk), na.rm=TRUE)
    t2sk<-sort(mk[ChoixTM==1,2], decreasing=TRUE)
    t2s<- max(t2sk[nos], min(t2sk), na.rm=TRUE)
    t1fk<-sort(mk[ChoixTM==0,1], decreasing=TRUE)
    t1f<-max(t1fk[nof*4], min(t1fk), na.rm=TRUE)
    t2fk<-sort(mk[ChoixTM==0,2], decreasing=TRUE)
    t2f<- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    
    #-------- Success probabilities --------------------------------------------------------
    # for sobral
    pass_sob <- (m1sk >= t1s & m2sk >= t2s )
    # for fortaleza
    pass_fort<- (m1sk >= t1f & m2sk >= t2f )
    #get them together
    c(pass_sob, pass_fort,t1s, t1f, t2s, t2f)
  }
  avps <- rowMeans(ps)
  # the success probabilities and thresholds average
  PSob[,i]<-avps[1:nobs]
  PFort[,i]<-avps[(nobs+1):(nobs*2)]
  tsim[i,]<-avps[(nobs*2+1):length(avps)]
  
  # convergence tracer
  convg[i]<-sum((PSob[,i] - probSIM[,1])^2 + (PFort[,i] - probSIM[,2])^2 > smallnumber )
  
  choix2 <- FUNchoiceTM(e1,e2,vecpref, cbind(PSob[,i],PFort[,i]), uboot)
  dc <- sum(abs(Choix[,1] - choix2[,1]) + abs(Choix[,2]-choix2[,2]), na.rm =TRUE)
  probSIM<-cbind(PSob[,i],PFort[,i]) 
  if (convg[i]< 10.5  )
    break
}
SIMsysTM<-list(convg, PSob, PFort,probSIM,choix2)

save(SIMsysTM, file="CMX_CountResult_TM.Rdata")
# stop the clustering clears the ram. remmeber to do it everytime
stopCluster(cl)
convg

# load the data if do not want to re-run the code
load("CMX_CountResult_TM.Rdata")

# convergence
CONVGTM <- SIMsysTM[[1]]
CONVGTM <- CONVGTM[CONVGTM!=0]   # check when does the convergence happen 

# the convergent success probabilities, which are later used to utility comparison exercise
PSIMTM <- SIMsysTM[[4]]

# choice at the convergence, the countefactual decision
dTM <- SIMsysTM[[5]]
 
# data for simulation of thresholds
datsimTM <- list(dTM, mft[,1], mft[,2])

simulTM <- matrix(runif(nobs*nsimT), nrow = nobs)   #random draw from uniform distribution

# RE-simulate the counterfactual systems to get the distribution of thresholds 
TSIMTMRESIM <- FUNresimTM(datsimTM,nsimT,ubootsamp,simulTM,PARgrade)
TTM <- apply(TSIMTMRESIM, 2, mean)
TTMsd <- apply(TSIMTMRESIM, 2, sd)


#####################################################################################
########################  utility comparison change #################################
#####################################################################################
attach(prep_boot)


Sob <- (Number ==48) # the choice in the original system 
# coefficient mu in (0.5, 1)
mu <- 0.8

# utility difference in the sub-population where both programs offer positive utility
UTTMSF <- exp(logvdiff)
# utility in sob > 0 and fort < 0 
UTTMSob <- exp(logvdiff + 0.5)*mu

# number of simulations used for computing expected utility
cl <- makeCluster(3)  # set the number of cpu you want to use
registerDoParallel(cl)

nobut <- 999

UTTMPOST <- foreach(i = (1:nobut),  .combine=cbind)%dopar%
{
  #  preference shock, once drawn, fix across all the iterations 
  eps1ct <-  runif(nobs)     #----- random draws for choice proportions
  eps2ct <-  runif(nobs)     #----- random draws for preference difference
  # simulate the timing change counterfactual
  e1<-eps1ct
  e2<-eps2ct
  # choices in each iteration
  choix<-FUNchoice(e1,e2,vecpref, PSIMTM, uboot)
  
  # individual utility in the two choice counterfactual system 
   PSIMTM[,1]*(UTTMSob*delta10/(delta10+delta11) + UTTMSF*delta11/(delta10+delta11))*Sob  + PSIMTM[,1]*1*(1-Sob)   
  
} 
stopCluster(cl)
utilpostTM<- apply(UTTMPOST,1,mean)

# individual utility in the original system 
utilorg <- p2sob*(UTTMSob*delta10/(delta10+delta11) + UTTMSF*delta11/(delta10+delta11))*Sob  + p2fort*1*(1-Sob)   
 
# the utility difference
utildifftm <- utilpostTM -utilorg

# the Final Grade -- the grade combination which is use to evalue the final admission
# this is taken as the ability proxy for the utility comparison exercise
FG <- 0.4*(m1p+120*Enem/63) + 0.6*m2p

#======= study the utility difference in difference percentage intervals =======================
#===============================================================================================

# get the percentiles of the final grades 
qtfg<-quantile(FG, probs=seq(0,1, 0.02))

# utility difference for all students
utildiffall<-rep(0,length(qtfg)-1)
utildiffallsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildiffall[i]<-mean( utildifftm[(FG>=qtfg[i] & FG<qtfg[i+1])]) # mean utility difference between (i-1)*% and i%
  utildiffallsd[i]<-sd( utildifftm[FG>=qtfg[i] & FG<qtfg[i+1]])   # stand deviation of utility difference between (i-1)*% and i%
}

# utility difference for those chose sobral 
utildiffsob<-rep(0,length(qtfg)-1)
utildiffsobsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildiffsob[i]<-mean( utildifftm[(FG>=qtfg[i] & FG<qtfg[i+1] & Sob==1)])
  utildiffsobsd[i]<-sd( utildifftm[FG>=qtfg[i] & FG<qtfg[i+1]  & Sob==1])
}

# utility difference for those chose fortaleza
utildifffort<-rep(0,length(qtfg)-1)
utildifffortsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildifffort[i]<-mean( utildifftm[(FG>=qtfg[i] & FG<qtfg[i+1] & Sob==0)])
  utildifffortsd[i]<-sd( utildifftm[FG>=qtfg[i] & FG<qtfg[i+1] & Sob==0])
}

#  utility difference in different intervals of interest 
D1<-c(mean( utildifftm[FG>=qtfg[1] & FG<qtfg[25+1]]),
      sd( utildifftm[FG>=qtfg[1] & FG<qtfg[25+1]]),
      mean( utildifftm[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==1]),
      sd( utildifftm[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==1]),
      mean( utildifftm[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==0]),
      sd( utildifftm[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==0])
)        # in the interval of below average -- in which the students have nearly 0 chance of being admitted

D2<-c(mean( utildifftm[FG>=qtfg[26] & FG<qtfg[30+1]]),
      sd( utildifftm[FG>=qtfg[26] & FG<qtfg[30+1]]),
      mean( utildifftm[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==1]),
      sd( utildifftm[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==1]),
      mean( utildifftm[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==0]),
      sd( utildifftm[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==0])
)       # from 50% to 60% 
D3<-c(mean( utildifftm[FG>=qtfg[31] & FG<qtfg[35+1]]),
      sd( utildifftm[FG>=qtfg[31] & FG<qtfg[35+1]]),
      mean( utildifftm[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==1]),
      sd( utildifftm[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==1]),
      mean( utildifftm[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==0]),
      sd( utildifftm[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==0])
)       # from 60% to 70%
D4<-c(mean( utildifftm[FG>=qtfg[36] & FG<qtfg[40+1]]),
      sd( utildifftm[FG>=qtfg[36] & FG<qtfg[40+1]]),
      mean( utildifftm[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==1]),
      sd( utildifftm[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==1]),
      mean( utildifftm[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==0]),
      sd( utildifftm[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==0])
)

Dcomp<-rbind(D1,D2,D3,D4, 
             cbind(utildiffall[41:50],utildiffallsd[41:50],
                   utildiffsob[41:50],utildiffsobsd[41:50],
                   utildifffort[41:50], utildifffortsd[41:50]))
xtable(Dcomp, digits=5)

sum(utildifftm[Sob==1]>0)/sum(Sob==1)
mean(utildifftm)
sd(utildifftm[Sob==1])



##############################################################################################################
#========================= Output of Adminssion rate change: Table and Figure ================================
##############################################################################################################

#========================== Table 6 ==========================================================================
T1 <- c(mean(utildifftm),NA,mean(utildifftm[Sob==1]),NA,mean(utildifftm[Sob==0]),NA)
T2 <- c(sd(utildifftm),NA,sd(utildifftm[Sob==1]),NA,sd(utildifftm[Sob==0]),NA)
T3 <- c(sum(utildifftm >0)/nobs,NA,sum(utildifftm[Sob==1]>0)/sum(Sob==1),NA,sum(utildifftm[Sob==0]>0)/sum(Sob==0),NA)

table_6 <- rbind(Dcomp, T1,T2,T3)

colnames(table_6) <- c("All mean","All s.d.","Sobral mean","Sobral s.d.","Fortaleza mean","Fortaleza s.d." )
rownames(table_6) <- c("0%-50%","50%-60%","60%-70%","70%-80%","80%-82%","82%-84%","84%-86%","86%-88%","88%-90%","90%-92%","92%-94%","94%-96%","96%-98%","98%-100%","Sample mean","Sample sd","Sample positive")

print(table_6, digits=2)


#========================= Figure 4 ==========================================================================
#probability change sob
pchangeSob <- PSIMTM[,1]-p2sob
changesob <- cbind(FG,pchangeSob)
pchangeSob1<- subset( changesob, Sob==1)
pchangeSob2<- subset( changesob, Sob==0)

#probability different plot
#pdf("spcTMsob_V1.pdf")
plot(FG, pchangeSob, ylim=c(-0.035,0.05),pch=1, cex=0.4,cex.axis=0.8, cex.lab=0.8, col='deepskyblue3', ann=FALSE)
lines(lowess(FG, pchangeSob, f =0.15, iter=5), lwd=2, col='red')
abline(h=0, lty=4, lwd=2, col='gray29') 
abline(v=qtfg[26], lty=4, col='orange')
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='blue')
title(ylab="Success probability change")
title(xlab="Expected final grade")
#dev.off()


#========================= Figure S.viii ==========================================================================
#probability change fortaleza
pchangeFort <- PSIMTM[,2]-p2fort
changefort <- cbind(FG,pchangeFort)

#pdf("spcTMfort_V1.pdf")
plot(FG, pchangeFort, ylim=c(-0.035,0.05),pch=1, cex=0.4, cex.axis=0.8, cex.lab=0.8, col='deepskyblue3', ann=FALSE)
lines(lowess(FG, pchangeFort, f =0.15, iter=5), lwd=2, col='red')
abline(v=qtfg[26], lty=4, col='orange')
abline(h=0, lty=4, lwd=2, col='gray29') 
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='blue')
title(ylab="Success probability change")
title(xlab="Expected final grade")
#dev.off()



#======================== Figure S.ix  ==========================================================================
mtm<-cbind(FG, utildifftm)
mtms<-subset(mtm, Sob==1)
mtmf<-subset(mtm, Sob==0)
# the plot 
#pdf("utctm_V1.pdf")
plot(mtmf[,1],mtmf[,2], pch=2, ylim=c(-0.01, 0.2),cex=0.4, col="deepskyblue3",ann=FALSE)
lines(lowess(mtmf[,1],mtmf[,2], f =0.15, iter=5), lwd=2, col='darkblue')
points(mtms[,1],mtms[,2], pch=5, cex=0.4, col="mistyrose3",ann=FALSE)
lines(lowess(mtms[,1],mtms[,2], f =0.15, iter=5), lty =6, lwd=2, col='salmon3')
abline(h=0, col='gray29')
abline(v=qtfg[26], lty=4, col='orange')
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='royalblue4')
title(xlab="Expected final grade")
title(ylab="Utility difference")
#dev.off()

