##############################################################################
######  choice generating function ###########################################
##############################################################################
 

FUNchoice<-function(eps1, eps2, vecpref, prob2, uboot)
{ 
  # load the parameters and vectors
  logvdiff <- vecpref[,1]    # log utility difference
  delta10 <- vecpref[,2]     # probability of only choosing Sobral
  delta11 <- vecpref[,3]     # probability of choosing both
  delta01 <- vecpref[,4]     # probability of only choosing Fortaleza
  ps <- vecpref[,5]          # mean success probability at Sobral in the original system
  pf <- vecpref[,6]          # mean success probability at Fortaleza in the original system
  Sob <- uboot[[3]]
  #------- simulate the epsilons ----------------------------------------------------------
  #epslion 1 conditional on Sobral
  eps1sob<-(delta10+delta11)*eps1  
  eps1sob<-eps1sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps1fort<-1-(delta10+delta11)*eps1
  eps1fort<-eps1fort[(Sob==0)]  
  # the simulated epsilon 1
  EPS1<-c(eps1sob,eps1fort)
  
  # epsilon 2
  # conditional on choosing Sobral
  eps2sob<-  -qnorm( pnorm(log(ps/pf)+logvdiff)*(1-eps2) )
  eps2sob<- eps2sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps2fort<-  qnorm( pnorm(-log(ps/pf)-logvdiff)*(1-eps2) )
  eps2fort<- eps2fort[(Sob==0)]
  EPS2<-c(eps2sob,eps2fort)
  
  # in case the probability is 0 
  logpd <- log(prob2[,1]/prob2[,2])
  logpd[prob2[,1]==0 & prob2[,2]==0] <- 1 
  logpd[prob2[,1]!=0 & prob2[,2]==0] <- 10
  
  # probability of choosing Sobral 
  ds <-ifelse(EPS1 < delta10, 1, ifelse(EPS1 > delta11+delta10, 0, 2))
  
  #probability of choose Sobral in the counterfactual 
  dm<-ifelse(logvdiff + logpd + EPS2>0, 1, 0)
  #-----------------------------------------------------------------------------
  ds[ds==2]<-dm[ds==2]
  #return the choice results
  
  return(ds)
}


########################################################################################
# choice function of two choice counterfactual 
FUNchoiceTC<-function(eps1, eps2, vecpref, prob2, uboot)
{
  # load the parameters and vectors
  logvdiff <- vecpref[,1]    # log utility difference
  delta10 <- vecpref[,2]     # probability of only choosing Sobral
  delta11 <- vecpref[,3]     # probability of choosing both
  delta01 <- vecpref[,4]     # probability of only choosing Fortaleza
  ps <- vecpref[,5]          # mean success probability at Sobral in the original system
  pf <- vecpref[,6]          # mean success probability at Fortaleza in the original system
  Sob <- uboot[[3]]
  #------- simulate the epsilons ----------------------------------------------------------
  #epslion 1 conditional on Sobral
  eps1sob<-(delta10+delta11)*eps1  
  eps1sob<-eps1sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps1fort<-1-(delta10+delta11)*eps1
  eps1fort<-eps1fort[(Sob==0)]  
  # the simulated epsilon 1
  EPS1<-c(eps1sob,eps1fort)
  
  # epsilon 2
  # conditional on choosing Sobral
  eps2sob<-  -qnorm( pnorm(log(ps/pf)+logvdiff)*(1-eps2) )
  eps2sob<- eps2sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps2fort<-  qnorm( pnorm(-log(ps/pf)-logvdiff)*(1-eps2) )
  eps2fort<- eps2fort[(Sob==0)]
  EPS2<-c(eps2sob,eps2fort)

  # probability of choosing Sobral only
  dsonly <-ifelse(EPS1 < delta10, 1, 0)
  # probability of choosing Fortaleza only 
  dfonly <-ifelse(EPS1 > delta11+delta10, 1, 0)
  #choosing both 
  d2c<-ifelse(EPS1>delta10 & EPS1<delta11+delta10,1,0)
  
  # in case the probability is 0 
  logpd <- log(prob2[,1]/prob2[,2])
  logpd[prob2[,1]==0 & prob2[,2]==0] <- 1 
  logpd[prob2[,1]!=0 & prob2[,2]==0] <- 10
  
  # 
  dsf<-ifelse( logvdiff + logpd + EPS2>0 & d2c==1, 1, 0)
  
  # probability of choosing (f, s)
  dfs<-ifelse( logvdiff + logpd + EPS2<0 & d2c==1, 1, 0)
  #-----------------------------------------------------------------------------
  #return the choice results
  choice<-cbind(dsonly, dfonly, dsf, dfs)
  return(choice)
}

#####################################################################################
## Choice function for timing change!! ############################################3

FUNchoiceTM<-function(eps1, eps2, vecpref, prob2, uboot)
{
  # load the parameters and vectors
  logvdiff <- vecpref[,1]    # log utility difference
  delta10 <- vecpref[,2]     # probability of only choosing Sobral
  delta11 <- vecpref[,3]     # probability of choosing both
  delta01 <- vecpref[,4]     # probability of only choosing Fortaleza
  ps <- vecpref[,5]          # mean success probability at Sobral in the original system
  pf <- vecpref[,6]          # mean success probability at Fortaleza in the original system
  Sob <- uboot[[3]]
  #------- simulate the epsilons ----------------------------------------------------------
  #epslion 1 conditional on Sobral
  eps1sob<-(delta10+delta11)*eps1
  eps1sob<-eps1sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps1fort<-1-(delta10+delta11)*eps1
  eps1fort<-eps1fort[(Sob==0)]  
  # the simulated epsilon 1
  EPS1<-c(eps1sob,eps1fort)
  
  # epsilon 2
  # conditional on choosing Sobral
  eps2sob<-  -qnorm( pnorm(log(ps/pf)+logvdiff)*(1-eps2) )
  eps2sob<- eps2sob[(Sob==1)]
  # conditional on choosing Fortaleza
  eps2fort<-  qnorm( pnorm(-log(ps/pf)-logvdiff)*(1-eps2) )
  eps2fort<- eps2fort[(Sob==0)]
  EPS2<-c(eps2sob,eps2fort)
  
  # probability of choosing Sobral only
  dsonly <-ifelse(EPS1 < delta10, 1, 0)
  # probability of choosing Fortaleza only 
  dfonly <-ifelse(EPS1 > delta11+delta10, 1, 0)
  #choosing both 
  d2c<-ifelse(EPS1>delta10 & EPS1<delta11+delta10,1,0)
  
  
  # in case the probability is 0 
  logpd <- log(prob2[,1]/prob2[,2])
  logpd[prob2[,1]==0 & prob2[,2]==0] <- 1 
  logpd[prob2[,1]!=0 & prob2[,2]==0] <- 10
  
  # probability of choosing (s, f)
  dsf<-ifelse(logvdiff + logpd + EPS2>0 & d2c==1, 1, 0)
  # probability of choosing (f, s)
  dfs<-ifelse(logvdiff + logpd + EPS2<0 & d2c==1, 1, 0)
  
  # choice for the timing change 
  choix1<-dsf+dsonly
  choix2<-rep(NA, nobs)
  choix2[dsf==1]<-0
  choix2[dfs==1]<-1 
  
  #-----------------------------------------------------------------------------
  #return the choice results
  choice<-cbind(choix1,choix2)
  return(choice)
}
