#========== The functions used for data imputation and computing thresholds ==========

#######################################################################################
#    imp:  the imputation function of the missing Enem, the ability proxy, also       #
#          applys the logistic transformation to have the standardrized ability       # 
#                                                                                     #
#    lgrad: the function that computes the limit passing grades                       #
#######################################################################################

#====================== Imputation of Enem when missing ==============================
imp <- function(don) {
  m0  <- log((Enem-6)/(64 -Enem))   #logistic transformation
  reg <- lm(m0 ~female+age+em_priv+rep+educf+educm+no_info, subset=(Enem>0))  
  e   <- rep(1, length(Enem)) 
  x   <- cbind(e,female,age,em_priv,rep,educf,educm,no_info)[Enem==0,]
  m0[Enem == 0] <- x %*% reg$coefficients  
  #  recover the imputed Enem
  m0m <- m0[Enem == 0]
  enem<- (64*exp(m0m)+6)/(exp(m0m)+1)
  
  mu0 <- mean(m0,na.rm = TRUE)
  m0[is.na(m0)] <- mu0
  return(list((m0 - mu0)/sd(m0) , enem))
}


####### Limit grades to pass #####
lgrad <- function(SIT1, SIT2) {
  
  ft <- factor(Number[substr(SIT1,1,1) == "A"])
  
  # Limit grade to pass: First stage
  m1a <- FS_FSE[substr(SIT1,1,1) == "A"]
  m1st <- tapply(m1a, ft, min)
  
  # Limit grade to pass: Second stage
  m2a <- FG[substr(SIT2,1,1) == "A"]
  ft2 <-  factor(Number[substr(SIT2,1,1) == "A"])
  m2st <- tapply(m2a, ft2, min)
  
  list(m1st, m2st)
}
