######################################################################################################################
###########   Functions of counterfactual simulations ################################################################
######################################################################################################################
# Function of Admission rate change counterfactual 
FUNsysARC<-function(e1,e2,u1,u2, mft,PARgrade,vecpref,uboot,NobARC)
{ 
  nsimul <- NobARC
  smallnumber <- 0.00001
  R <- 2
  probSIM <- vecpref[,c(5,6)]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  m1h <- mft[,1]
  m2h <- mft[,2]
  Enem <- uboot[[4]]
  PSob <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  PFort <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  tsim <- matrix(rep(0), nrow=nsimul, ncol=4)
  convg <- rep(0, NobARC)
  for (i in 1:NobARC)
  { 
    # the choice  
    Sobsim <- FUNchoice(e1,e2, vecpref,probSIM,uboot)
    # the simulated grades
    m1s <- m1h + u1[,i] 
    # the combination for ranking
    m1sk <- m1s + 120*Enem/63
    # the second stage grades 
    m2s <- m2h + cfu1m2*u1[,i] + u2[,i] 
    # the combination for ranking
    m2sk <- 0.4*m1sk + 0.6*m2s
    # divide the population by their choices in the sample
    # chances are that the people chose Sobral is small than the admission number
    t1sk <- sort(m1sk[Sobsim==1], decreasing=TRUE)
    t1s  <- max(t1sk[nos*R], min(t1sk),na.rm=TRUE)
    t1fk <- sort(m1sk[Sobsim==0], decreasing=TRUE) 
    t1f  <- max(t1fk[nof*R], min(t1sk),na.rm=TRUE)
    # second stage
    t2sk <- sort(m2sk[Sobsim==1 & m1sk>=t1s], decreasing=TRUE)
    t2s  <- max(t2sk[nos], min(t2sk), na.rm=TRUE) 
    t2fk <- sort(m2sk[Sobsim==0 & m1sk>=t1f], decreasing=TRUE)
    t2f  <- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    tsim[i,] <- c(t1s, t1f, t2s, t2f)
    
    # Individual invariants 
    bm10 <- rep(1,nobs) %o% c(t1s,t1f)  - 120*Enem/63 - m1h
    bm20 <- rep(1,nobs) %o% c(t2s,t2f)/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
    #  Success probabilities 
    ps <- cbind( sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1), sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1))
    PSob[,i]  <- ps[,2]  # the new success probabilities at soral
    PFort[,i] <- ps[,4]  # the new success probabilities at fortaleza
    
    #  convergence criterion 
    convg[i] <- sum((PSob[,i] - probSIM[,1])^2 + (PFort[,i] - probSIM[,2])^2 > smallnumber )
    probSIM <- cbind(PSob[,i],PFort[,i]) 
    if (convg[i] < 0.5 )
      break
  }
  return(list(tsim, convg, PSob,PFort,probSIM,Sobsim))
}

####################################################################################


# Function of two choice counterfactual 
FUNsysTC<-function(e1,e2,uboot,mft,PARgrade,vecpref,NobTC)
{
  uboot <- ubootsamp
  e1<-eps1ct
  e2<-eps2ct
  nsimul <- 200
  nresim <- NobTC
  smallnumber <- 0.00001
  R <- 4
  probSIM <- vecpref[,c(5,6)]
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  m1h <- mft[,1]
  m2h <- mft[,2]
  Enem <- uboot[[4]]
  
  PSob <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  PFort <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  tsim <- matrix(rep(0), nrow=nsimul, ncol=4)
  SobSIM <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  convg <- rep(0, nsimul)
  
  # start the iteration 
  for (i in 1:nsimul)
  { 
    choix<-FUNchoiceTC(e1,e2,vecpref, probSIM, uboot)
    sobonly<-choix[,1]
    fortonly<-choix[,2]
    dsf<-choix[,3]
    dfs<-choix[,4]
    
    u1 <- sample(uboot[[1]], size = nobs*nresim, replace=TRUE)
    u2 <- sample(uboot[[2]], size = nobs*nresim, replace=TRUE) 
    
    u1 <- matrix(u1, nrow=nobs, ncol=nresim)
    u2 <- matrix(u2, nrow =nobs, ncol=nresim)
    # simulate the success probabilities 
    ps<-foreach(j=(1:nresim), .combine="cbind") %dopar%
{
    
    # the simulated grades
    m1s<-m1h+u1[,j]  
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1[,j] + u2[,j]  
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # divide the population by their choices in the sample
    # chances are that the people chose Sobral is small than the admission number
    t1sk<-sort(m1sk[sobonly==1 | dsf==1], decreasing=TRUE)
    t1s<- max(t1sk[nos*R], min(t1sk),na.rm=TRUE)
    t1fk<-sort(m1sk[fortonly==1 | dfs==1], decreasing=TRUE) 
    t1f<- max(t1fk[nof*R], min(t1sk),na.rm=TRUE)
    # refinement of the thresholds. 
    if (t1s > t1f){
      #take those who fail sobral when choosing (s,f)
      nrankfort<-c(m1sk[fortonly==1 | dfs==1], m1sk[dsf==1 & m1sk<t1s])
      nrankfort<-sort(nrankfort, decreasing=TRUE)
      t1f<-max(nrankfort[nof*R], min(nrankfort),na.rm=TRUE)
    }else{
      #take those who fail sobral when choosing (f,s)
      nranksob<-c(m1sk[sobonly==1 | dsf==1 ], m1sk[dfs==1 & m1sk<t1f])
      nranksob<-sort(nranksob, decreasing=TRUE)
      t1s<-max(nranksob[nos*R], min(nranksob),na.rm=TRUE)
    }
    #sob
    sob2t<-ifelse(sobonly==1& m1sk>= t1s, 1, 0)
    sob22t<-ifelse(dsf==1 & m1sk>=t1s,1,0)
    sob222t<- ifelse(dfs==1 & m1sk>=t1s & m1sk<t1f,1,0)
    sum(sob22t+sob2t+sob222t)
    sobtwo<-sob22t+sob2t+sob222t
    #fort
    fort2t<-ifelse(fortonly==1 & m1sk >= t1f, 1, 0)
    fort22t<-ifelse(dfs==1 & m1sk>=t1f,1,0)
    fort222t<-ifelse(dsf ==1 & m1sk>=t1f & m1sk<t1s, 1, 0)
    sum(fort222t+fort22t+fort2t)
    forttwo<- fort222t+fort22t+fort2t
    
    # second stage
    t2sk<-sort(m2sk[sobtwo==1], decreasing=TRUE)
    t2s<- max(t2sk[nos], min(t2sk), na.rm=TRUE) 
    t2fk<-sort(m2sk[forttwo==1], decreasing=TRUE)
    t2f<- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    
    #-------- Individual invariants --------------------------------------------------------
    # new thresholds are obtained then move on to 
    bm10 <- rep(1,nobs) %o% c(t1s,t1f)  - 120*Enem/63 - m1h
    bm20 <- rep(1,nobs) %o% c(t2s,t2f)/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
    #-------- Success probabilities --------------------------------------------------------
    # for sobral
    sucsob<-sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1)
    # for fortaleza
    sucfort<-sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1)
    #get them together
    c(sucsob[,2],sucfort[,2])
    }
    avps <- rowMeans(ps)
   
    PSob[,i]<-avps[1:nobs]
    PFort[,i]<-avps[(nobs+1):(nobs*2)]
     
    convg[i]<-sum((PSob[,i] - probSIM[,1])^2 + (PFort[,i] - probSIM[,2])^2 > smallnumber )
    probSIM<-cbind(PSob[,i],PFort[,i]) 
    if (convg[i] <  0.5 )
      break
  }
  return(list(convg, PSob, PFort,probSIM,choix))
}

##################################################################################################

# the function of timing change counterfactual 
#simple version of the function
FUNsysTM<-function(e1,e2,u1,u2, mft,PARgrade,vecpref,uboot,NobTM)
{
  nsimul <- NobTM
  smallnumber <- 0.0001
  R <- 4
  probSIM <- cbind(vecpref[,5]*4, vecpref[,5], vecpref[,6]*4, vecpref[,6])
  cfu1 <- PARgrade[[3]]
  cfu1m2 <- PARgrade[[2]][9]
  m1h <- mft[,1]
  m2h <- mft[,2]
  Enem <- uboot[[4]]
  
  PSob <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  PFort <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  tsim <- matrix(rep(0), nrow=nsimul, ncol=4)
  SobSIM <- matrix(rep(0),nrow=nobs, ncol=nsimul)
  convg <- rep(0, nsimul)
  tsim[1,]<-c(185,193,237.8,240.8)
  
  for (i in 1:nsimul)
  { 
    probcont<-probSIM[,c(2,4)]/probSIM[,c(1,3)]
    Choix<-FUNchoiceTM(e1,e2,vecpref, probcont, uboot)
    # the simulated thresholds    
    # the simulated grades
    m1s<-m1h+u1[,i] 
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1*u1[,i] +u2[,i] 
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    
    # one by one decision 
    Nsob<-0
    Nfort<-0
    ChoixTM<-rep(0,nobs)
    # all the variables 
    mk<-cbind(m1sk, m2sk,Choix)
    mk<-mk[order(mk[,1], decreasing=TRUE), ]  # sort the grade wrt m1
    
    for (j in 1:nobs){
      #one by one decision making
      if (Nsob<nos*4 & Nfort<nof*4){
        # proc 1
        ChoixTM[j]<-Choix[j,1]
      }
      if (Nsob<nos*4 & Nfort == nof*4){
        #proc 2
        if (Choix[j,1]==1){
          ChoixTM[j]<-Choix[j,1]
        }
        else{
          ChoixTM[j]<-Choix[j,2]
        } 
      }
      if (Nsob==nos*4 & Nfort < nof*4){
        # proc 3
        if (Choix[j,1]==1){
          ChoixTM[j]<-Choix[j,2]
        }
        else{
          ChoixTM[j]<-Choix[j,1]
        } 
      }
      if (Nsob==nos*4 & Nfort==nof*4){
        # proc 4
        ChoixTM[j]<-NA
      } 
      ChoixTM[j]
      # addition of Nsob
      if (is.na(ChoixTM[j])==FALSE){
        Nsob<-Nsob+ChoixTM[j]
      }else{
        Nsob<-Nsob
      }
      # addition of Nfort
      if (is.na(ChoixTM[j])==FALSE){
        Nfort<-Nfort+(1-ChoixTM[j])
      }else{
        Nfort<-Nfort
      }
      
    }
    # the final program choice
    SobSIM[,i]<-ChoixTM
    
    # second stage
    t1sk<-sort(mk[ChoixTM==1,1], decreasing=TRUE)
    t1s<-max(t1sk[nos*4], min(t1sk), na.rm=TRUE)
    t2sk<-sort(mk[ChoixTM==1,2], decreasing=TRUE)
    t2s<- max(t2sk[nos], min(t2sk), na.rm=TRUE)
    t1fk<-sort(mk[ChoixTM==0,1], decreasing=TRUE)
    t1f<-max(t1fk[nof*4], min(t1fk), na.rm=TRUE)
    t2fk<-sort(mk[ChoixTM==0,2], decreasing=TRUE)
    t2f<- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    tsim[i,]<-c(t1s, t1f, t2s, t2f)
    
    #-------- Individual invariants --------------------------------------------------------
    # new thresholds are obtained then move on to 
    bm10 <- rep(1,nobs) %o% c(t1s,t1f)  - 120*Enem/63 - m1h
    bm20 <- rep(1,nobs) %o% c(t2s,t2f)/.6  - 2*(m1h + 120*Enem/63)/3 - as.vector(m2h)
    #-------- Success probabilities --------------------------------------------------------
    ps <- cbind( sucprob(cbind(bm10[,1], bm20[,1]),uboot,simul,cfu1), sucprob(cbind(bm10[,2], bm20[,2]),uboot,simul,cfu1))
     
    PSob[,i]<-ps[,2] 
    PFort[,i]<-ps[,4] 
    
    convg[i]<-sum((PSob[ ,i] - probSIM[ ,2])^2 + (PFort[ ,i] - probSIM[ ,4])^2 > smallnumber )
    probSIM <- ps
    if (convg[i] < 0.5 )
      break
  }
  return(list(tsim, convg, PSob,PFort,probSIM,Choix))
}

