# Subroutines: Truncated probit as simulated choice model for Magnac-Xiong

# Simulated data 
simulTP <- function(coef0) {
  
  # variables
  u <- cbind(matrix(runif(nobs*3), nobs, 3), rnorm(nobs))
  w <- sb + (ub-sb)*u[,1]
  m0 <- -.5 + u[,2]
  xdt <-  m0
  xa <- m0
  
  # Global invariants
  dt1 <- 1/(1+exp(-cbind(rep(1,nobs), xdt) %*% coef0[1:ndt]))
  dt <- (1-dt1)/(1+exp(-cbind(rep(1,nobs), xdt) %*% coef0[(ndt+1):(2*ndt)]))
  a <- cbind(rep(1,nobs), xa) %*% coef0[(2*ndt+1):(2*ndt+na)]
  
  # Dependent variable
  d <- (u[,3] < dt1) + (u[,3] > 1-dt)*(a+w > u[,4])
  
  # Simulated data
  cbind(d, w, xdt, xa)
}

# Likelihood function 
likTP <- function(coef) {
  
  delta1 <- 1/(1+exp(-cbind(rep(1,nobs), dta[,3:(ndt+1)]) %*% coef[1:ndt]))
  delta <- (1-delta1)/(1+exp(-cbind(rep(1,nobs), dta[,3:(ndt+1)]) %*% coef[(ndt+1):(2*ndt)]))
  arg <- cbind(rep(1,nobs), dta[,(ndt+2):(ndt+na)]) %*% coef[(2*ndt+1):(2*ndt+na)]
  
  p <- delta1 + delta*pnorm(arg + dta[,2]) 
  
  dta[,1]*log(p) + (1-dta[,1])*log(1-p) - lam*sum(coef^2)/2
}

slikTP <- function(coef) sum(-likTP(coef))

#========================== Likelihood gradient========================================================
gradlikTPe <- function(coef) {
  
  lind <- list(1:ndt, ndt+(1:ndt), 2*ndt+(1:na))
  ldta <- list(cbind(rep(1,nobs), dta[,3:(ndt+1)]), cbind(rep(1,nobs), dta[,3:(ndt+1)]), 
               cbind(rep(1,nobs), dta[,(ndt+2):(ndt+na)]))
  
  # Parameters in each index 
  narg <- paste("likTP", "(", "c", "(", sep="")
  pderiv <- NULL
  
  m <- 0
  while (m < 3) {
    m <- m + 1
    
    nom <- paste("c", m, sep="")
    assign(nom, coef[lind[[m]]][1])
    narg <- paste(narg, nom, ",", sep="") 
    pderiv <- c(pderiv, nom)
    
    nom <- paste("cr", m, sep="")
    assign(nom, coef[lind[[m]]][-1])
    
    if (m<3) narg <- paste(narg, nom, ",", sep="") else narg <- paste(narg, nom, "))", sep="")
    
  }
  
  # Principal gradient 
  g <- numericDeriv(quote(eval(parse(text=narg))), pderiv)
  grad <- attr(g,"gradient")
  
  
  # Gradient recomposition
  m <- 0
  grf <- NULL
  while (m < 3) {
    m <- m + 1
    grf <- cbind(grf, (grad[,m] + lam*coef[lind[[m]]][1])*ldta[[m]] 
                 - lam * rep(1,nobs) %o% coef[lind[[m]]])
  }
  
  grf
  
}  

gradlikTP <- function(coef) apply(-gradlikTPe(coef), 2, sum)